'
'
' ####################
' #####  PROLOG  #####
' ####################
'
PROGRAM "NewYear2018"
VERSION "1.00"
'STRIP		' strip unreferenced internal FUNCTIONs
'CONSOLE
'
' NewYear2018 - New Year 2018 the XBLite Way
' Copyright (c) GPL 2017 Guy Lonne.
'
' ------------------------------------------------------------------------------
' The following source code is released without restrictions.
' ------------------------------------------------------------------------------
'
' ***** Description *****
' Happy new year 2018, the XBLite way!
'
'
' ***** Versions *****
' 1.00-23dec17-Guy-Generated GUI prototype using viXen v1.99u.
' Operating System: Windows 10
' Generation switches that are on:
' - Use Windows API
' - Use Windows XP Style
' - Use Debugging Functions
' - Use Text-only Push Buttons
' - Meticulous Cleanup
'
'
' ##############################
' #####  Import Libraries  #####
' ##############################
'
' WinAPI DLL headers
'
	IMPORT "kernel32"    ' Operating System
'
' ---Note: import gdi32 BEFORE shell32 and user32
	IMPORT "gdi32"       ' Graphic Device Interface
	IMPORT "shell32"     ' interface to Operating System
	IMPORT "user32"      ' Windows management
'
' ---Note: import comctl32 BEFORE comdlg32
	IMPORT "comctl32"    ' common controls; ==> initialize w/ InitCommonControlsEx ()
	IMPORT "winmm"       ' Windows MultiMedia
'
' XBLite headers
'
	IMPORT "xst"         ' XBLite Standard Library
	IMPORT "xsx"         ' XBLite Standard eXtended Library
'	IMPORT "xio"         ' console library
	IMPORT "lcdpanel"    ' LCD Panel
'
'
'
' ****************************************
' *****  COMPOSITE TYPE DEFINITIONS  *****
' ****************************************
'
' dialog box template used in NewDialogBox()
'
TYPE TDLGDATA
	DLGTEMPLATE .dltt
	USHORT      .menu
	USHORT      .class
	USHORT      .title
END TYPE
'
'
'
' ********************************************
' *****  INTERNAL FUNCTION DECLARATIONS  *****
' ********************************************
'
DECLARE FUNCTION Entry () ' program entry point

DECLARE FUNCTION CleanUp () ' program cleanup

DECLARE FUNCTION CreateWindows () ' create the windows of the program


DECLARE FUNCTION GuiTellApiError (msg$) ' display a WinAPI error message

DECLARE FUNCTION InitGui () ' initialize win32 controls and libraries
DECLARE FUNCTION InitWindows () ' for initializations after CreateWindows()

DECLARE FUNCTION KillFont (@hFont) ' release the logical font

DECLARE FUNCTION MessageLoop () ' main message loop

DECLARE FUNCTION NewChild (class$, text$, style, x, y, w, h, hParent, idCtr, exStyle) ' create a child window
DECLARE FUNCTION NewDialogBox (hParent, width, height, addrProc, initParam) ' create a dialog
DECLARE FUNCTION NewFont (fontName$, pointSize, weight, bItalic, bUnderL, bStrike) ' create a new logical font
DECLARE FUNCTION NewWindow (class$, titleBar$, style, x, y, w, h, exStyle) ' create a window

DECLARE FUNCTION RegisterWinClass (class$, addrProc, icon$, menu$) ' register the window class

DECLARE FUNCTION SetCtrText (hCtr, text$) ' set the text of a control
DECLARE FUNCTION SetFont (hCtr, hFont) ' apply font to control and redraw it
DECLARE FUNCTION SetFontAndRedraw (hCtr, hFont, bRedraw) ' apply font to control, eventually redraw it

DECLARE FUNCTION StartUp () ' program setup

DECLARE FUNCTION WndProc (hWnd, wMsg, wParam, lParam) ' callback function for window #winMain

DECLARE FUNCTION dlgAbout_callback (hWnd, wMsg, wParam, lParam) ' callback function for dialog #dlgAbout
'
'
' *****************************************
' *****  SHARED CONSTANT DEFINITIONS  *****
' *****************************************
'
'
'
' ***** Constants used as control identificators for window #winMain *****
'
' control identificators for #winMain
'
$$ledPanel               = 101	' lcd panel
$$mleMsg                 = 102	' multiline editor
$$btnAbout               = 103	' push button '&About'
$$btnExit                = 104	' push button 'E&xit'
'
' menu identificators for Window #winMain
'
$$mnuMain                = 8000		' menubar
'
$$mnuFile                = 8001		' sub-menu "&File"
$$mnuHelp                = 8002		' sub-menu "&Help"
'
$$mnuFileExit            = 8003		' menu option "E&xit"
'
$$mnuHelpAbout           = 8008		' menu option "&About\tCtrl+F1"
'
'
'
' ***** Constants used as control identificators for dialog #dlgAbout *****
'
$$dlgAbout               = 300	' dialog 'About NewYear2018'
'
' control identificators for #dlgAbout
'
$$dlgAbout_lblTitle      = 301	' static '\"" + PROGRAM$ (0) + "\" - New Year 2018 the XBLite Way'
$$dlgAbout_lblVer        = 302	' static 'Version " + VERSION$ (0) + "'
$$dlgAbout_lblDisclaimer = 303	' static 'Copyright (c) GPL 2017 Guy Lonne.'
$$dlgAbout_mleDesc       = 304	' multiline editor 'Happy'
$$dlgAbout_btnClose      = 305	' push button '&Close'
'
'
' ***** Shared Program Constants *****
'
'
'
' ******************************************
' *****  SHARED VARIABLE DECLARATIONS  *****
' ******************************************
'
'
' WINDOWS GUI SHARED VARIABLES
'
SHARED hInst		' global instance handle
SHARED hFontDefault ' standard font
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
' Program entry point.
'
FUNCTION Entry ()
	STATIC entry            ' ensure Entry() is entered only one time

	IF entry THEN RETURN    ' enter once...
	entry = $$TRUE          ' ...and then no more

'	XioCreateConsole ("", 50)		' create console

	bErr = StartUp ()		' initialize program and libraries
	IF bErr THEN XstAlert ("Entry: Can't initialize the program")

	bErr = InitGui ()		' initialize the GUI
	IF bErr THEN XstAlert ("Entry: Can't initialize the GUI")

	bErr = CreateWindows ()		' create the windows of the program
	IF bErr THEN XstAlert ("Entry: Can't create the windows of the program")

	InitWindows ()          ' for initializations after CreateWindows()
	MessageLoop ()          ' the message loop
	CleanUp ()              ' program cleanup

'	a$ = INLINE$ ("Press ENTER to exit >")
'	XioFreeConsole ()		' free console
	QUIT (0)

END FUNCTION
'
'
' ########################
' #####  CleanUp ()  #####
' ########################
'
' Program cleanup.
'
FUNCTION CleanUp ()
	SHARED hInst		' global instance handle
	SHARED hFontDefault ' standard font

	' 1.Release the images

	IF #winMain_image THEN
		SetLastError (0)
		ret = DeleteObject (#winMain_image)		' release the bitmap #winMain_image
		IFZ ret THEN
			msg$ = "DeleteObject: Can't release the bitmap #winMain_image"
			GuiTellApiError (msg$)
		ENDIF
		'
	ENDIF
	#winMain_image = 0

	' 2.Release the logical fonts

	KillFont (@#MS_Sans_Serif_48)
	KillFont (@#comic_sans_ms_12)
	KillFont (@hFontDefault)

	' 3.Destroy the accelerator tables

	IF #hAccel THEN DestroyAcceleratorTable (#hAccel)
	#hAccel = 0

	' 4.Destroy dialogs and windows

	IF #dlgAbout THEN EndDialog (#dlgAbout, 0)		' ending dialog $$dlgAbout
	#dlgAbout = 0

	IF #winMain THEN
		SetLastError (0)
		ret = DestroyWindow (#winMain)		' destroy window #winMain
		IFZ ret THEN
			msg$ = "DestroyWindow: Can't destroy window #winMain"
			GuiTellApiError (msg$)
		ENDIF
		'
	ENDIF
	#winMain = 0

	' 5.Unregister window class(es)

	SetLastError (0)
	ret = UnregisterClassA (&#winMain_class$, hInst)		' unregister window class #winMain_class$
	IFZ ret THEN
		msg$ = "UnregisterClassA: Can't unregister window class '" + #winMain_class$ + "'"
		GuiTellApiError (msg$)
	ENDIF

END FUNCTION
'
'
' ##############################
' #####  CreateWindows ()  #####
' ##############################
'
' Creates the windows of the program.
'
FUNCTION CreateWindows ()
	SHARED hInst

	' ***************** Begin window #winMain setup *****************

	' register the window class for window #winMain
	#winMain_class$ = PROGRAM$ (0) + "_winMain"
	addrProc = &WndProc ()
	RegisterWinClass (#winMain_class$, addrProc, "0", "")		' register window class #winMain_class$
	image$ = "violinImg"
	SetLastError (0)
	#winMain_image = LoadBitmapA (hInst, &image$)		' load image violin.bmp from the resource
	IFZ #winMain_image THEN
		msg$ = "LoadBitmapA: Can't load image violin.bmp from the resource"
		GuiTellApiError (msg$)
	ENDIF

	' create window #winMain
	titleBar$ = "Happy New Year 2018, the XBLite way!"
	#winMain = NewWindow (#winMain_class$, titleBar$, $$WS_SYSMENU, 0, 0, 760, 530, 0)		' create window #winMain
	IFZ #winMain THEN
		msg$ = "NewWindow: Can't create window #winMain"
		XstAlert (msg$)
		RETURN $$TRUE		' fail
	ENDIF

	' ***************** End window #winMain setup *****************

	XstCenterWindow (#winMain)		' auto-center the window
	ShowWindow (#winMain, $$SW_SHOWNORMAL)		' show the window

END FUNCTION
'
' Displays a WinAPI error message.
' Returns bErr: $$TRUE only if an error REALLY occurred.
'
' Usage:
'	SetLastError (0)
'	hImage = LoadImageA (0, &file$, $$IMAGE_BITMAP, 0, 0, $$LR_LOADFROMFILE)
'	IFZ hImage THEN
'		msg$ = "LoadImageA: Can't load image file " + file$
'		bErr = GuiTellApiError (msg$)
'		IF bErr THEN RETURN $$TRUE		' fail
'	ENDIF
'
FUNCTION GuiTellApiError (msg$)

	' get the last error code, then clear it
	errNum = GetLastError ()
	SetLastError (0)
	IFZ errNum THEN RETURN		' was OK!

	fmtMsg$ = "Last error code " + STRING$ (errNum) + ": "

	' set up FormatMessageA arguments
	dwFlags = $$FORMAT_MESSAGE_FROM_SYSTEM | $$FORMAT_MESSAGE_IGNORE_INSERTS
	sizeBuf = 1020 : szBuf$ = NULL$ (sizeBuf)
	ret = FormatMessageA (dwFlags, 0, errNum, 0, &szBuf$, sizeBuf, 0)
	IFZ ret THEN
		fmtMsg$ = fmtMsg$ + "(unknown)"
	ELSE
		fmtMsg$ = fmtMsg$ + CSTRING$ (&szBuf$)
	ENDIF

	IFZ msg$ THEN msg$ = "Windows API error"

	' get the OS name and version
	XstGetOSName (@osName$)
	XstGetOSVersion (@major, @minor, @platformId, @version$, @platform$)

	fmtMsg$ = fmtMsg$ + $$CRLF$ + $$CRLF$ + msg$ + $$CRLF$ + "OS: " + osName$ + " ver." + STR$ (major) + "." + STRING$ (minor) + "-" + platform$

	? fmtMsg$		' output message to an active console

	title$ = PROGRAM$ (0) + "-API Error"
	hwnd = GetActiveWindow ()
	MessageBoxA (hwnd, &fmtMsg$, &title$, $$MB_ICONSTOP)

	RETURN $$TRUE		' an error really occurred!

END FUNCTION
'
'
' ########################
' #####  InitGui ()  #####
' ########################
'
' Initializes the User Interface.
'
' Initializes win32 controls and libraries.
' Returns bErr: $$TRUE on fail.
'
FUNCTION InitGui ()
	SHARED hInst
	SHARED hFontDefault		' default control font

	INITCOMMONCONTROLSEX iccex

	SetLastError (0)
	IFZ hInst THEN
		hInst = GetModuleHandleA (0)		' a valid hInst ensures proper images loading
		IFZ hInst THEN
			msg$ = "InitGui: Can't get current instance handle"
			GuiTellApiError (msg$)
		ENDIF
	ENDIF

	' $$ICC_ANIMATE_CLASS      : animate
	' $$ICC_BAR_CLASSES        : toolbar, status bar, trackbar, tooltips
	' $$ICC_COOL_CLASSES       : rebar (coolbar) control
	' $$ICC_DATE_CLASSES       : month picker, date picker, time picker, up-down
	' $$ICC_HOTKEY_CLASS       : hotkey
	' $$ICC_INTERNET_CLASSES   : WIN32_IE >= 0x0400
	' $$ICC_LISTVIEW_CLASSES   : listview, header
	' $$ICC_PAGESCROLLER_CLASS : page scroller (WIN32_IE >= 0x0400)
	' $$ICC_PROGRESS_CLASS     : progress bar
	' $$ICC_TAB_CLASSES        : tab control, tooltips
	' $$ICC_TREEVIEW_CLASSES   : treeview, tooltips
	' $$ICC_UPDOWN_CLASS       : up-down
	' $$ICC_USEREX_CLASSES     : extended combo box
	' $$ICC_WIN95_CLASSES      : everything else

	' initialize the common control classes
	iccex.dwSize = SIZE (iccex)

	iccex.dwICC = $$ICC_ANIMATE_CLASS      | _
              $$ICC_BAR_CLASSES        | _
              $$ICC_COOL_CLASSES       | _
              $$ICC_DATE_CLASSES       | _
              $$ICC_HOTKEY_CLASS       | _
              $$ICC_INTERNET_CLASSES   | _
              $$ICC_LISTVIEW_CLASSES   | _
              $$ICC_NATIVEFNTCTL_CLASS | _
              $$ICC_PAGESCROLLER_CLASS | _
              $$ICC_PROGRESS_CLASS     | _
              $$ICC_TAB_CLASSES        | _
              $$ICC_TREEVIEW_CLASSES   | _
              $$ICC_UPDOWN_CLASS       | _
              $$ICC_USEREX_CLASSES     | _
              $$ICC_WIN95_CLASSES

	InitCommonControlsEx (&iccex)		' initialize comctl32.dll library

	SetLastError (0)
	hFontDefault = GetStockObject ($$ANSI_VAR_FONT)		' get the ANSI default font
	IFZ hFontDefault THEN		' fail
		msg$ = "InitGui: Can't get the ANSI default font"
		GuiTellApiError (msg$)
	ENDIF

	LCDPanel ()		' initialize LCD Panel
	#comic_sans_ms_12 = NewFont ("Comic Sans MS", 12, $$FW_NORMAL, $$FALSE, $$FALSE, $$FALSE)		' create logical font #comic_sans_ms_12
	IFZ #comic_sans_ms_12 THEN
		msg$ = "NewFont: Can't create logical font #comic_sans_ms_12"
		XstAlert (msg$)
	ENDIF

	#MS_Sans_Serif_48 = NewFont ("Microsoft Sans Serif", 48, $$FW_NORMAL, $$FALSE, $$FALSE, $$FALSE)		' create logical font #MS_Sans_Serif_48
	IFZ #MS_Sans_Serif_48 THEN
		msg$ = "NewFont: Can't create logical font #MS_Sans_Serif_48"
		XstAlert (msg$)
	ENDIF

END FUNCTION
'
'
' ############################
' #####  InitWindows ()  #####
' ############################
'
' Initializes after CreateWindows().
'
FUNCTION InitWindows ()

	DIM #text$[3]
'	#text$[0] = "Guy Lonne and his family wish you a happy new year!"
	#text$[0] = "For you and your family, I wish you a happy new year!"
	#text$[1] = "May love, health and wealth brighten your lives here,"
	#text$[2] = "So that you can shine in your Community,"
	#text$[3] = "And twinkle like stars in the Human Galaxy."

	FOR index = 0 TO UBOUND (#text$[])
		IFZ index THEN
			text$ = #text$[index]
		ELSE
			text$ = text$ + $$CRLF$ + #text$[index]
		ENDIF
	NEXT index
	SetCtrText (#mleMsg, text$)

	UpdateWindow (#winMain)		' Update window #winMain

END FUNCTION
'
' Releases a logical font.
'
' r_hFont = handle to the logical font, reset on exit.
' Returns bOK: $$TRUE if success.
'
FUNCTION KillFont (@r_hFont)

	SetLastError (0)

	IF r_hFont THEN
		DeleteObject (r_hFont)		' free memory space
	ELSE
		msg$ = "KillFont: Ignore a NULL font handle"
		XstAlert (msg$)
		RETURN $$TRUE		' fail
	ENDIF

	r_hFont = 0

END FUNCTION
'
' The main message loop.
'
' Receives and handles messages sent to it from the system in reaction
' to User interactions and system events.
'
' Returns $$FALSE on receiving a QUIT message or $$TRUE on error.
'
FUNCTION MessageLoop ()
	MSG msg		' will be sent to window callback function when an event occurs

	' supervise system messages until
	' - the User decides to leave the application (RETURN $$FALSE)
	' - an error occurred (returns bErr: $$TRUE on error)
	DO
		' retrieve next message from queue
		ret = GetMessageA (&msg, 0, 0, 0)
		SELECT CASE ret
			CASE 0 : RETURN		' QUIT message
			CASE -1 : RETURN $$TRUE		' fail
			CASE ELSE
				' deal with window messages
				hwnd = GetActiveWindow ()
				'
				' Process accelerator keys for menu commands
				ret = 0
				hAccel = 0
				IF hwnd = #winMain THEN hAccel = #hAccel
				IF hAccel THEN ret = TranslateAcceleratorA (hwnd, hAccel, &msg)
				IFZ ret THEN
					IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN
						' send only non-dialog messages
						' translate virtual-key messages into character messages
						' ex.: SHIFT + a is translated as "A"
						TranslateMessage (&msg)
						'
						' send message to window callback function
						DispatchMessageA (&msg)
					ENDIF
				ENDIF		' accelerator key
		END SELECT
	LOOP		' forever

END FUNCTION
'
' Creates a child window.
'
FUNCTION NewChild (class$, text$, style, x, y, w, h, hParent, idCtr, exStyle)
	SHARED hInst
	SHARED hFontDefault

	' create the child window
	newStyle = style | $$WS_CHILD | $$WS_VISIBLE

	SetLastError (0)
	hCtr = CreateWindowExA (exStyle, &class$, &text$, newStyle, x, y, w, h, hParent, idCtr, hInst, 0)
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create the child window '" + text$ + "', control class '" + class$ + "'"
		GuiTellApiError (msg$)
		RETURN		' fail
	ENDIF

	SendMessageA (hCtr, $$WM_SETFONT, hFontDefault, 1)		' redraw
	RETURN hCtr

END FUNCTION
'
' Creates a dialog.
'
FUNCTION NewDialogBox (hParent, width, height, addrProc, initParam)

	SHARED hInst
	TDLGDATA tdd		' dialog template (Composite type defined in this program)
	DLGTEMPLATE dltt
	XLONG bu_x
	XLONG bu_y

	' fill in DLGTEMPLATE struct
	dltt.style = $$DS_MODALFRAME | $$DS_3DLOOK | $$WS_POPUP | $$WS_CAPTION | $$WS_SYSMENU | $$WS_VISIBLE | $$DS_ABSALIGN | $$DS_SETFONT
	dltt.dwExtendedStyle = 0
	dltt.cdit = 0		' number of control items in dialog, this creates an empty dialog

	screenWidth = GetSystemMetrics ($$SM_CXSCREEN)
	IF width < 0 THEN width = 0
	x = (screenWidth - width) >> 1		' center dialog on screen

	screenHeight = GetSystemMetrics ($$SM_CYSCREEN)
	IF height < 0 THEN height = 0
	y = (screenHeight - height) >> 1

	bu = GetDialogBaseUnits ()		' get dialog base units
	bu_x = LOWORD (bu)
	bu_y = HIWORD (bu)

	' convert from pixels to dialog units
	dltt.x = (x * 4) / bu_x		' left in dialog units
	dltt.y = (y * 8) / bu_y		' top  in dialog units
	dltt.cx = (width * 4) / bu_x		' width  in dialog units
	dltt.cy = (height * 8) / bu_y		' height in dialog units

	' fill in TDLGDATA struct
	tdd.dltt = dltt
	tdd.menu = 0
	tdd.class = 0
	tdd.title = 0

	' create dialog
	RETURN DialogBoxIndirectParamA (hInst, &tdd, hParent, addrProc, initParam)

END FUNCTION
'
' Creates a logical font.
'
' Returns the handle to the new font or 0 on fail.
'
FUNCTION NewFont (fontName$, pointSize, weight, bItalic, bUnderL, bStrike)
	LOGFONT oLogFont

	SetLastError (0)
	r_hFont = 0

	' check fontName$ not empty
	fontName$ = TRIM$ (fontName$)
	IFZ fontName$ THEN
		msg$ = "NewFont: empty font face"
		XstAlert (msg$)
		RETURN		' fail
	ENDIF

	' hFontToClone provides with a well-formed font structure
	SetLastError (0)
	hFontToClone = GetStockObject ($$DEFAULT_GUI_FONT)		' get a font to clone
	IFZ hFontToClone THEN
		msg$ = "GetStockObject: Can't get a font to clone"
		bErr = GuiTellApiError (msg$)
		IF bErr THEN RETURN		' invalid handle
	ENDIF

	bErr = $$FALSE
	SetLastError (0)
	bytes = GetObjectA (hFontToClone, SIZE (LOGFONT), &oLogFont)		' fill allocated structure oLogFont
	IF bytes <= 0 THEN
		msg$ = "GetObjectA: Can't fill allocated structure oLogFont"
		bErr = GuiTellApiError (msg$)
		IF bErr THEN RETURN		' invalid handle
	ENDIF

	' release the cloned font
	DeleteObject (hFontToClone) : hFontToClone = 0		' free memory space

	' set the cloned font structure with the passed parameters
	oLogFont.faceName = fontName$

	IFZ pointSize THEN
		oLogFont.height = 0
	ELSE
		' character height is specified (in points)
		pointH = pointSize
		IF pointH < 0 THEN pointH = -pointH		' make it positive
		'
		' convert pointSize to pixels
		SetLastError (0)
		hdc = GetDC ($$HWND_DESKTOP)		' get a handle to the desktop context
		IFZ hdc THEN
			msg$ = "GetDC: Can't get a handle to the desktop context"
			bErr = GuiTellApiError (msg$)
			IF bErr THEN RETURN		' invalid handle
		ENDIF
		'
		' Windows expects the font height to be in pixels and negative
		oLogFont.height = MulDiv (pointH, GetDeviceCaps (hdc, $$LOGPIXELSY), -72)
		ReleaseDC ($$HWND_DESKTOP, hdc)		' release the handle to the desktop context
	ENDIF

	SELECT CASE weight
		CASE $$FW_THIN, $$FW_EXTRALIGHT, $$FW_LIGHT, $$FW_NORMAL, $$FW_MEDIUM, _
			$$FW_SEMIBOLD, $$FW_BOLD, $$FW_EXTRABOLD, $$FW_HEAVY, $$FW_DONTCARE
			oLogFont.weight = weight
		CASE ELSE
			oLogFont.weight = $$FW_NORMAL
	END SELECT

	IF bItalic THEN oLogFont.italic = 1 ELSE oLogFont.italic = 0
	IF bUnderL THEN oLogFont.underline = 1 ELSE oLogFont.underline = 0
	IF bStrike THEN oLogFont.strikeOut = 1 ELSE oLogFont.strikeOut = 0

	SetLastError (0)
	r_hFont = CreateFontIndirectA (&oLogFont)		' create logical font r_hFont
	IFZ r_hFont THEN
		msg$ = "CreateFontIndirectA: Can't create logical font r_hFont"
		bErr = GuiTellApiError (msg$)
		IF bErr THEN RETURN		' invalid handle
	ENDIF

	RETURN r_hFont

END FUNCTION
'
' Creates a window.
'
FUNCTION NewWindow (class$, titleBar$, style, x, y, w, h, exStyle)
	SHARED hInst

	SetLastError (0)
	hWnd = CreateWindowExA (exStyle, &class$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)
	IFZ hWnd THEN
		msg$ = "NewWindow: Can't create the window, window class " + class$
		GuiTellApiError (msg$)
	ENDIF

	RETURN hWnd

END FUNCTION
'
' Registers the window class.
' Returns bErr: $$TRUE on fail.
'
FUNCTION RegisterWinClass (class$, addrProc, icon$, menu$)
	SHARED hInst

	WNDCLASSEX wc		' window class

	XstGetOSVersion (@major, @minor, @platformId, @version$, @platform$)
	IF (major > 5) && (major < 10) THEN
		' Vista, Windows 7 and 8
		wc.hbrBackground = $$COLOR_WINDOW
	ELSE
		wc.hbrBackground = $$COLOR_BTNFACE + 1
	ENDIF

	wc.style = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc = addrProc
	wc.cbSize = SIZE (WNDCLASSEX)
	wc.cbClsExtra = 0		' no extra bytes after the window class
	wc.cbWndExtra = 0		' structure or the window instance
	wc.hInstance = hInst

	IF icon$ THEN wc.hIcon = LoadIconA (hInst, &icon$)

	wc.hCursor = LoadCursorA (0, $$IDC_ARROW)
	wc.lpszMenuName = 0
	IF menu$ THEN wc.lpszMenuName = &menu$
	wc.lpszClassName = &class$

	SetLastError (0)
	ret = RegisterClassExA (&wc)
	IFZ ret THEN
		msg$ = "RegisterWinClass: Can't register the window class '" + class$ + "'"
		GuiTellApiError (msg$)
		RETURN $$TRUE		' fail
	ENDIF

END FUNCTION
'
' Sets the text for a control.
' hCtr = control handle
' text$ = text to set
' Returns bOK: $$TRUE on success.
'
FUNCTION SetCtrText (hCtr, text$)

	SetLastError (0)
	bOK = $$FALSE

	IF hCtr THEN
		ret = SetWindowTextA (hCtr, &text$)
		IF ret THEN bOK = $$TRUE
	ENDIF

	RETURN bOK

END FUNCTION
'
' Wrapper to SetFontAndRedraw.
'
FUNCTION SetFont (hCtr, hFont)

	bOK = SetFontAndRedraw (hCtr, hFont, $$TRUE)		' redraw
	RETURN bOK

END FUNCTION
'
' Applies a font to control.
'
' hCtr    = control handle
' hFont   = logical font handle
' bRedraw = $$TRUE to redraw the control after setting the font
' Returns bOK: $$TRUE on success.
'
FUNCTION SetFontAndRedraw (hCtr, hFont, bRedraw)

	SetLastError (0)
	bOK = $$FALSE

	SELECT CASE hCtr
		CASE 0
			msg$ = "SetFontAndRedraw: Ignore a NULL control handle"
			XstAlert (msg$)
			'
		CASE ELSE
			' check hFont not null
			IFZ hFont THEN
				msg$ = "SetFontAndRedraw: Ignore a NULL font handle"
				XstAlert (msg$)
				EXIT SELECT		' fail
			ENDIF
			'
			IF bRedraw THEN lParam = 1 ELSE lParam = 0
			SendMessageA (hCtr, $$WM_SETFONT, hFont, lParam)		' $$WM_SETFONT does not return a value
			bOK = $$TRUE
			'
	END SELECT

	RETURN bOK

END FUNCTION
'
'
' ########################
' #####  StartUp ()  #####
' ########################
'
' Program Setup.
'
' Returns bErr: $$TRUE on fail.
'
FUNCTION StartUp ()
	SHARED hInst		' global instance handle

	SetLastError (0)
	hInst = GetModuleHandleA (0)		' get current instance handle
	IFZ hInst THEN
		msg$ = "GetModuleHandleA: Can't get current instance handle"
		bErr = GuiTellApiError (msg$)
		IF bErr THEN RETURN $$TRUE		' fail
	ENDIF

	LCDPanel ()		' activate LCD Panel

END FUNCTION
'
' Callback function for window #winMain.
'
FUNCTION WndProc (hWnd, wMsg, wParam, lParam)
	SHARED hInst		' global instance handle

	XLONG notifyCode, idCtr, hCtr

	RECT rect
	PAINTSTRUCT ps
	BITMAP bitmap
	ACCEL accel[]		' accelerator table

	DIM accel[0]		' 1 accelerators

	SELECT CASE wMsg
		CASE $$WM_CREATE		' event thrown by CreateWindowExA
			GOSUB winMain_Create
' timer+++
			SetTimer (hWnd, 1, 300, 0)						' set timer id 1
			RETURN
			'
		CASE $$WM_TIMER
			'SetCtrText (#winMain, STR$ (#TicTac))
			IFZ #TicTac THEN GOSUB PlayTune
			INC #TicTac
			IF #TicTac > 160 THEN #TicTac = 0
			'
			IFZ #line$ THEN
				#line$ = "         Happy New Year 2018!"		' first time
			ELSE
				#line$ = LCLIP$ (#line$) + LEFT$ (#line$)		' roll text to the left
			ENDIF
			SetCtrText (#ledPanel, #line$)
			RETURN
' timer~~~
		CASE $$WM_PAINT
			hdc = BeginPaint (hWnd, &ps)
			IF #winMain_image THEN
				' add a bitmap to the window
				hdcImage = CreateCompatibleDC (hdc)
				hbmOld = SelectObject (hdcImage, #winMain_image)
				bytes = 0		' number of bytes stored into the buffer
				SetLastError (0)
				bytes = GetObjectA (#winMain_image, SIZE (BITMAP), &bitmap)		' allocate the structure BITMAP
				IF bytes <= 0 THEN
					msg$ = "GetObjectA: Can't allocate the structure BITMAP"
					GuiTellApiError (msg$)
				ENDIF
				'
				x = 0
				y = 0
				BitBlt (hdc, x, y, bitmap.width, bitmap.height, hdcImage, 0, 0, $$SRCCOPY)
				SelectObject (hdcImage, hbmOld)
				DeleteDC (hdcImage)		' free memory space
			ENDIF
			'
			EndPaint (hWnd, &ps)
			RETURN
			'
		CASE $$WM_CLOSE		' closed by User
			DestroyWindow (hWnd)		' triggers $$WM_DESTROY
			RETURN
			'
		CASE $$WM_DESTROY		' being destroyed
			PostQuitMessage ($$WM_QUIT)		' quit program
			#winMain = 0
			RETURN
			'
		CASE $$WM_SIZE
			bErr = $$FALSE
			SetLastError (0)
			ret = GetWindowRect (hWnd, &rect)		' get rectangle of the window
			IFZ ret THEN
				msg$ = "GetWindowRect: Can't get rectangle of the window"
				bErr = GuiTellApiError (msg$)
				IF bErr THEN RETURN		' invalid handle
			ENDIF
			'
			winLeft = rect.left
			winTop = rect.top
			winWidth = rect.right - rect.left
			winHeight = rect.bottom - rect.top
			'
			bResize = $$FALSE
			' ( 760, 530 ) = minimum width and height of the window
			IF winWidth < 760 THEN
				' increase above the minimum width
				winWidth = 760
				bResize = $$TRUE
			ENDIF
			'
			IF winHeight < 530 THEN
				' increase above the minimum height
				winHeight = 530
				bResize = $$TRUE
			ENDIF
			'
			IFF bResize THEN RETURN
			'
			SetWindowPos (hWnd, $$HWND_TOP, 0, 0, winWidth, winHeight, $$SWP_NOMOVE)
			'
			RETURN
			'
		CASE $$WM_COMMAND
' normal---
'			notifyCode = HIWORD (wParam)
'			idCtr = LOWORD (wParam)
' normal~~~
' speedy+++
			' Split wParam into its high and low parts.
ASM	mov eax,d[WndProc.wParam]
ASM	mov	ebx,65536
ASM	cdq
ASM	idiv	ebx
ASM	mov d[WndProc.notifyCode],eax ; = HIWORD
ASM	mov d[WndProc.idCtr],edx      ; = LOWORD
' speedy~~~
			hCtr = lParam
			'
			SELECT CASE idCtr
				CASE $$mnuFileExit
					DestroyWindow (hWnd)		' triggers $$WM_DESTROY
					RETURN
					'
				CASE $$mnuHelpAbout
					addrProc = &dlgAbout_callback ()
					NewDialogBox (#winMain, 624, 361, addrProc, 0)		' creating dialog $$dlgAbout
					RETURN
					'
				CASE $$btnAbout
					IF notifyCode = $$BN_CLICKED THEN
						addrProc = &dlgAbout_callback ()
						NewDialogBox (#winMain, 624, 361, addrProc, 0)		' creating dialog $$dlgAbout
						RETURN
					ENDIF
					'
				CASE $$btnExit
					IF notifyCode = $$BN_CLICKED THEN
						DestroyWindow (hWnd)		' triggers $$WM_DESTROY
						RETURN
					ENDIF
					'
			END SELECT
			'
	END SELECT

	RETURN DefWindowProcA (hWnd, wMsg, wParam, lParam)		' message not handled

SUB PlayTune

	' play the music
	sound_file$ = "NewYear2018.mid"
	'cmd$ = "PLAY " + sound_file$

	XstPathToAbsolutePath (sound_file$, @fullPath$)
	cmd$ = "PLAY " + fullPath$

	SetLastError (0)
	ret = mciExecute (&cmd$)
	IFZ ret THEN		' fail
		msg$ = "InitWindows: Can't play '" + sound_file$ + "'"
		GuiTellApiError (msg$)
	ENDIF

END SUB

SUB winMain_Create

	#winMain = hWnd		' set the window global handle

	' *********** Begin Menu setup ***********

	iaccel = -1		' accelerator index
	SetLastError (0)
	#mnuMain = CreateMenu ()		' create window #winMain's menubar
	IFZ #mnuMain THEN
		msg$ = "CreateMenu: Can't create window #winMain's menubar"
		GuiTellApiError (msg$)
	ENDIF

	SetLastError (0)
	mnuFile = CreateMenu ()		' create dropdown sub-menu mnuFile
	IFZ mnuFile THEN
		msg$ = "CreateMenu: Can't create dropdown sub-menu mnuFile"
		GuiTellApiError (msg$)
	ENDIF

	SetLastError (0)
	ret = AppendMenuA (#mnuMain, $$MF_POPUP, mnuFile, &"&File")		' insert sub-menu mnuFile
	IFZ ret THEN
		msg$ = "AppendMenuA: Can't insert sub-menu mnuFile"
		GuiTellApiError (msg$)
	ENDIF

	SetLastError (0)
	ret = AppendMenuA (mnuFile, $$MF_STRING, $$mnuFileExit, &"E&xit")		' add menu option $$mnuFileExit
	IFZ ret THEN
		msg$ = "AppendMenuA: Can't add menu option $$mnuFileExit"
		GuiTellApiError (msg$)
	ENDIF

	SetLastError (0)
	mnuHelp = CreateMenu ()		' create dropdown sub-menu mnuHelp
	IFZ mnuHelp THEN
		msg$ = "CreateMenu: Can't create dropdown sub-menu mnuHelp"
		GuiTellApiError (msg$)
	ENDIF

	SetLastError (0)
	ret = AppendMenuA (#mnuMain, $$MF_POPUP, mnuHelp, &"&Help")		' insert sub-menu mnuHelp
	IFZ ret THEN
		msg$ = "AppendMenuA: Can't insert sub-menu mnuHelp"
		GuiTellApiError (msg$)
	ENDIF

	SetLastError (0)
	ret = AppendMenuA (mnuHelp, $$MF_STRING, $$mnuHelpAbout, &"&About\tCtrl+F1")		' add menu option $$mnuHelpAbout
	IFZ ret THEN
		msg$ = "AppendMenuA: Can't add menu option $$mnuHelpAbout"
		GuiTellApiError (msg$)
	ENDIF

	INC iaccel
	IF iaccel > UBOUND (accel[]) THEN
		msg$ = "window #winMain: acceleration table too small; iaccel: " + STRING$ (iaccel)
		XstAlert (msg$)
		REDIM accel[iaccel]
	ENDIF

	accel[iaccel].cmd = $$mnuHelpAbout
	accel[iaccel].fVirt = $$FVIRTKEY | $$FCONTROL
	accel[iaccel].key = $$VK_F1

	SetLastError (0)
	ret = SetMenu (#winMain, #mnuMain)		' activate the menubar
	IFZ ret THEN
		msg$ = "SetMenu: Can't activate the menubar"
		GuiTellApiError (msg$)
	ENDIF

	' initialize the accelerator table
	cEntries = UBOUND (accel[]) + 1
	SetLastError (0)
	#hAccel = CreateAcceleratorTableA (&accel[0], cEntries)		' create accelerator table #hAccel
	IFZ #hAccel THEN
		msg$ = "CreateAcceleratorTableA: Can't create accelerator table #hAccel"
		GuiTellApiError (msg$)
	ENDIF

	' *********** End Menu setup ***********

	' *********** Begin Controls setup ***********

	' creating lcd panel $$ledPanel
	SetLastError (0)
	#ledPanel = CreateDisplayPanel (#winMain, "", $$ledPanel, 140, 250, 560, 80, 1)
	IFZ #ledPanel THEN
		msg$ = "CreateDisplayPanel: Can't create lcd panel $$ledPanel"
		GuiTellApiError (msg$)
	ENDIF

	ConfigureDisplayPanel (#ledPanel, 1, 10, "blue", 0, 0, 0, 1)

	' creating multiline editor $$mleMsg
	#mleMsg = NewChild ($$EDIT, "", $$ES_MULTILINE, 232, 370, 418, 100, #winMain, $$mleMsg, $$WS_EX_CLIENTEDGE)		' create multiline editor $$mleMsg
	IFZ #mleMsg THEN
		msg$ = "NewChild: Can't create multiline editor $$mleMsg"
		XstAlert (msg$)
	ENDIF

	SetFont (#mleMsg, #comic_sans_ms_12)

	' creating push button $$btnAbout
	hCtr = NewChild ($$BUTTON, "&About", $$WS_TABSTOP, 12, 446, 76, 24, #winMain, $$btnAbout, 0)		' create push button $$btnAbout
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create push button $$btnAbout"
		XstAlert (msg$)
	ENDIF

	' creating push button $$btnExit
	hCtr = NewChild ($$BUTTON, "E&xit", $$WS_TABSTOP, 100, 446, 76, 24, #winMain, $$btnExit, 0)		' create push button $$btnExit
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create push button $$btnExit"
		XstAlert (msg$)
	ENDIF

	' *********** End Controls setup ***********

END SUB		' winMain_Create

END FUNCTION
'
' Callback function for dialog #dlgAbout.
'
' Returns 1 on message handled
'
FUNCTION dlgAbout_callback (hWnd, wMsg, wParam, lParam)
	SHARED hInst		' global instance handle

	XLONG notifyCode, idCtr, hCtr

	SELECT CASE wMsg
		CASE $$WM_INITDIALOG
			GOSUB dlgAbout_Create
			#dlgAbout_Canceled = $$FALSE		' dialog not canceled
			'
			' set dialog icon with the program's icon
			icon$ = "0"
			SetLastError (0)
			hIcon = LoadIconA (hInst, &icon$)		' load icon scrabble.ico
			IFZ hIcon THEN
				msg$ = "LoadIconA: Can't load icon scrabble.ico"
				GuiTellApiError (msg$)
			ENDIF
			'
			IF hIcon THEN
				' set the icon in title bar
				SendMessageA (hWnd, $$WM_SETICON, $$ICON_BIG, hIcon)
				SendMessageA (hWnd, $$WM_SETICON, $$ICON_SMALL, hIcon)
				DeleteObject (hIcon) : hIcon = 0		' free memory space
			ENDIF
			XstCenterWindow (hWnd)
			RETURN 1		' keyboard focus on default control
			'
		CASE $$WM_CLOSE
			pRet = 0		' pointer to returned value
			EndDialog (hWnd, pRet)		' destroy dialog
			#dlgAbout = 0
			#dlgAbout_Canceled = $$TRUE		' dialog canceled
			'
		CASE $$WM_COMMAND
' normal---
'			notifyCode = HIWORD (wParam)
'			idCtr = LOWORD (wParam)
' normal~~~
' speedy+++
			' Split wParam into its high and low parts.
ASM	mov eax,d[dlgAbout_callback.wParam]
ASM	mov	ebx,65536
ASM	cdq
ASM	idiv	ebx
ASM	mov d[dlgAbout_callback.notifyCode],eax ; = HIWORD
ASM	mov d[dlgAbout_callback.idCtr],edx      ; = LOWORD
' speedy~~~
			hCtr = lParam
			'
			SELECT CASE idCtr
				CASE $$dlgAbout_btnClose, $$IDCANCEL
					IF notifyCode = $$BN_CLICKED THEN
						pRet = 0		' pointer to returned value
						EndDialog (hWnd, pRet)		' destroy dialog
						#dlgAbout = 0
						#dlgAbout_Canceled = $$TRUE		' dialog canceled
						RETURN 1
					ENDIF		' clicked
					'
			END SELECT		' CASE idCtr
			'
	END SELECT

	RETURN		' message not handled

SUB dlgAbout_Create

	#dlgAbout = hWnd		' set the dialog global handle

	titleBar$ = "About NewYear2018"
	SetWindowTextA (hWnd, &titleBar$)		' set dialog title

	' *********** Begin Controls setup ***********

	' creating static $$dlgAbout_lblTitle
	text$ = "\"" + PROGRAM$ (0) + "\" - New Year 2018 the XBLite Way"
	hCtr = NewChild ($$STATIC_CLASS, text$, 0, 12, 12, 250, 20, #dlgAbout, $$dlgAbout_lblTitle, 0)		' create static $$dlgAbout_lblTitle
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create static $$dlgAbout_lblTitle"
		XstAlert (msg$)
	ENDIF

	' creating static $$dlgAbout_lblVer
	text$ = "Version " + VERSION$ (0) + ""
	hCtr = NewChild ($$STATIC_CLASS, text$, 0, 12, 32, 250, 20, #dlgAbout, $$dlgAbout_lblVer, 0)		' create static $$dlgAbout_lblVer
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create static $$dlgAbout_lblVer"
		XstAlert (msg$)
	ENDIF

	' creating static $$dlgAbout_lblDisclaimer
	text$ = "Copyright (c) GPL 2017 Guy Lonne."
	hCtr = NewChild ($$STATIC_CLASS, text$, 0, 12, 52, 250, 56, #dlgAbout, $$dlgAbout_lblDisclaimer, 0)		' create static $$dlgAbout_lblDisclaimer
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create static $$dlgAbout_lblDisclaimer"
		XstAlert (msg$)
	ENDIF

	' creating multiline editor $$dlgAbout_mleDesc
	style = $$ES_MULTILINE | $$ES_AUTOHSCROLL | $$ES_READONLY | $$WS_HSCROLL | $$WS_VSCROLL
	text$ = "Happy\r\nNew\r\nYear\r\n2018!"
	#dlgAbout_mleDesc = NewChild ($$EDIT, text$, style, 270, 12, 340, 340, #dlgAbout, $$dlgAbout_mleDesc, $$WS_EX_CLIENTEDGE)		' create multiline editor $$dlgAbout_mleDesc
	IFZ #dlgAbout_mleDesc THEN
		msg$ = "NewChild: Can't create multiline editor $$dlgAbout_mleDesc"
		XstAlert (msg$)
	ENDIF

	SetFont (#dlgAbout_mleDesc, #MS_Sans_Serif_48)

	' creating push button $$dlgAbout_btnClose
	hCtr = NewChild ($$BUTTON, "&Close", $$WS_TABSTOP, 100, 326, 76, 24, #dlgAbout, $$dlgAbout_btnClose, 0)		' create push button $$dlgAbout_btnClose
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create push button $$dlgAbout_btnClose"
		XstAlert (msg$)
	ENDIF

	' *********** End Controls setup ***********

END SUB		' dlgAbout_Create

END FUNCTION

END PROGRAM
